import pygame, copy, player, random
from constants import *
from function import *
import menu as menu_module

WIDTH = 640
HEIGHT = 480
WHITE = (255,255,255)
LIGHT_BLUE = (100,100,200)
BLACK = (0,0,0)

class Player(object):
    
    def __init__(self, mapsize, start_point, walls, flow_lines, switches, keys, colour, nick, i):
        self.rect = pygame.Rect(0, 0, PLAYER_SIZE, PLAYER_SIZE)
        self.rect.center = start_point[0]*FIELD_SIZE+FIELD_SIZE/2, start_point[1]*FIELD_SIZE+FIELD_SIZE/2
        self.float_pos = [float(self.rect.x), float(self.rect.y)] 
        self.walls = walls
        self.flow_lines = flow_lines
        self.switches = switches
        self.key = keys
        self.colour = colour
        self.alive = True
        self.score = 0
        self.nick = nick
        self.last_time = 0
        self.went = 0         # here is how many pixels our player went in real
        self.went_prec = 0.0  # here is how many pixels our player should take, that means e.g. went = 4, went_prec = 4.6
        self.direction = 0
        self.frame = 0
        self.img_list = load_player_image_list(i)
        self.map_width = mapsize[0]
        self.map_height = mapsize[1]
        self.switch_collide = 0
    
    def update(self, key_list, time):
        xmove = ymove = 0
        up = left = down = right = False 
        if key_list[0] and not key_list[2]:
            up = True
        elif key_list[2] and not key_list[0]:
            down = True
        if key_list[1] and not key_list[3]:
            left = True
        if key_list[3] and not key_list[1]:
            right = True
        delta = (time - self.last_time)*0.001
        if right or left or up or down:
            self.went_prec += PLAYER_SPEED*delta
            if round(self.went_prec) > self.went: #right now we move!
                step = round(self.went_prec) - self.went
                self.went = int(round(self.went_prec))
                if (right or left) and (up or down):
                    step = step/SQRT2
                if up:
                    ymove = -step
                    self.direction = 0
                if left:
                    xmove = -step
                    self.direction = 2
                if down:
                    ymove = step
                    self.direction = 4
                if right:
                    xmove = step
                    self.direction = 6
                if up and left:
                    self.direction = 1
                elif left and down:
                    self.direction = 3
                elif down and right:
                    self.direction = 5
                elif right and up:
                    self.direction = 7
                self.frame = (self.went % (7*IMG_CHANGE)) / IMG_CHANGE
                if self.frame == 3: self.frame = 1
                elif self.frame == 4: self.frame = 0
                elif self.frame == 5: self.frame = 3
                elif self.frame == 6: self.frame = 4
                elif self.frame == 7: self.frame = 3
        else:
            self.frame = 0
        self.move(xmove, ymove, delta)         
        
        if self.switch_collide:
            self.switch_collide -= 1    
        self.last_time = time

    def get_image(self):
        return self.img_list[self.direction][self.frame]

    def move(self, dx, dy, time):
        collides = []
        for flow_line in self.flow_lines:
            if self.rect.colliderect(flow_line.rect):
                collides.append(flow_line)
        c = []
        for flow_line in collides:
            if flow_line.direction not in c:
                moved = True
                move = flow_line.get_move(time)
                dx += move[0]
                dy += move[1]
                c.append(flow_line.direction)

        if dx != 0:
            self.__move(dx, 0)
        if dy != 0:
            self.__move(0, dy)
    
    def __move(self, dx, dy):
        self.float_pos[0] += dx
        self.float_pos[1] += dy
        self.rect.x = int(round(self.float_pos[0])) 
        self.rect.y = int(round(self.float_pos[1]))
        
        for wall in self.walls:
            if self.rect.colliderect(wall.rect):
                if dx > 0:
                    self.rect.right = wall.rect.left
                    self.float_pos[0] = float(self.rect.x)
                if dx < 0:
                    self.rect.left = wall.rect.right
                    self.float_pos[0] = float(self.rect.x)
                if dy > 0:
                    self.rect.bottom = wall.rect.top
                    self.float_pos[1] = float(self.rect.y)
                if dy < 0:
                    self.rect.top = wall.rect.bottom
                    self.float_pos[1] = float(self.rect.y)
        for switch in self.switches:
            if self.rect.colliderect(switch.rect):
                if dx > 0:
                    self.rect.right = switch.rect.left
                    self.float_pos[0] = float(self.rect.x)
                if dx < 0:
                    self.rect.left = switch.rect.right
                    self.float_pos[0] = float(self.rect.x)
                if dy > 0:
                    self.rect.bottom = switch.rect.top
                    self.float_pos[1] = float(self.rect.y)
                if dy < 0:
                    self.rect.top = switch.rect.bottom
                    self.float_pos[1] = float(self.rect.y)
                if not self.switch_collide:
                    switch.collide()
                self.switch_collide = 6
        if self.rect.x < 0:
            self.rect.x = 0
            self.float_pos[0] = float(self.rect.x)
        if self.rect.right > self.map_width - 1:
            self.rect.right = self.map_width - 1
            self.float_pos[0] = float(self.rect.x)
        if self.rect.y < 0:
            self.rect.y = 0
            self.float_pos[1] = float(self.rect.y)
        if self.rect.bottom > self.map_height - 1:
            self.rect.bottom = self.map_height - 1
            self.float_pos[1] = float(self.rect.y)
                    
class FlowLine:
    def __init__(self, position, speed, direction):
        self.image = load_image("flow_line.png")
        self.orig_img = self.image.copy()
        if direction[0] == -1:
            self.image = pygame.transform.rotate(self.image, 90)
        elif direction[1] == 1:
            self.image = pygame.transform.rotate(self.image, 180)
        elif direction[0] == 1:
            self.image = pygame.transform.rotate(self.image, 270)
        self.img1 = self.image.copy()
        self.img2 = self.image.copy()
        self.rect = self.img1.get_rect()  #main rect
        self.rect1 = self.img1.get_rect() #rect of the first image
        self.rect2 = self.img2.get_rect() #rect of the second image
        self.rect.topleft = position[0]*FIELD_SIZE,position[1]*FIELD_SIZE
        self.rect1.topleft = 0,0
        self.rect2.topleft = 0-FIELD_SIZE*direction[0],0-FIELD_SIZE*direction[1]
        self.cur_dist = 0
        self.speed = speed
        self.direction = direction
        self.last_time = 0
        self.went = 0
        self.went_prec = 0.0
        self.update(pygame.time.get_ticks())
        self.position = position
    
    def set_direction(self, d):
        if d == "^":
            self.direction = (0,-1)
            self.image = self.orig_img.copy()
            self.img1 = self.image.copy()
            self.img2 = self.image.copy()
        elif d == "<":
            self.direction = (-1,0)
            self.image = pygame.transform.rotate(self.orig_img, 90)
            self.img1 = self.image.copy()
            self.img2 = self.image.copy()
        elif d == "v":
            self.direction = (0,1)
            self.image = pygame.transform.rotate(self.orig_img, 180)
            self.img1 = self.image.copy()
            self.img2 = self.image.copy()
        elif d == ">":
            self.direction = (1,0)
            self.image = pygame.transform.rotate(self.orig_img, 270)
            self.img1 = self.image.copy()
            self.img2 = self.image.copy()
        self.rect1.topleft = 0,0
        self.rect2.topleft = 0-FIELD_SIZE*self.direction[0],0-FIELD_SIZE*self.direction[1]
        self.cur_dist = 0
        self.went = 0
        self.went_prec = 0.0
        
    def set_speed(self, s):
        self.speed = s

    def update(self, time):
        delta = (time - self.last_time)*0.001
        self.went_prec += self.speed*delta
        step = 0
        if round(self.went_prec) > self.went: #right now we move!
            step = round(self.went_prec) - self.went
            self.went = int(round(self.went_prec))
        self.cur_dist += step
        if self.cur_dist > FIELD_SIZE:
            self.cur_dist -= FIELD_SIZE
            self.rect1.x -= FIELD_SIZE*self.direction[0]
            self.rect1.y -= FIELD_SIZE*self.direction[1]
            self.rect2.x -= FIELD_SIZE*self.direction[0]
            self.rect2.y -= FIELD_SIZE*self.direction[1]
        self.rect1.x += step*self.direction[0]
        self.rect1.y += step*self.direction[1]
        self.rect2.x += step*self.direction[0]
        self.rect2.y += step*self.direction[1]
        self.image.blit(self.img1, self.rect1)
        self.image.blit(self.img2, self.rect2)

        self.last_time = time
        
    def get_move(self, delta):
        return [self.speed*delta*self.direction[0], self.speed*delta*self.direction[1]]
        
class Wall:
    def __init__(self, x, y, shape="-"):
        if shape == "-":
            self.image = pygame.transform.rotate(load_image("block.png"), 90)
            self.rect = self.image.get_rect()
            self.rect.topleft = x*FIELD_SIZE, ((y+1)*FIELD_SIZE)-(self.rect.height/2)
        else:
            self.image = load_image("block.png")
            self.rect = self.image.get_rect()
            self.rect.topleft = ((x+1)*FIELD_SIZE)-(self.rect.width/2), y*FIELD_SIZE
            
class Bench:
    def __init__(self, x, y):
        self.image = load_image("bench.png", None, True)
        self.rect = self.image.get_rect()
        self.rect.topleft = (x*FIELD_SIZE, y*FIELD_SIZE)
            
class Switch:
    def __init__(self, x, y, t, coords, values, flow_lines):
        if not coords:
            self.env = "g" #global
        else:
            self.env = "l" #local
        self.img1 = load_image("paka1.png", None, True)
        self.img2 = load_image("paka2.png", None, True)
        self.image = self.img2.copy()
        self.rect = self.image.get_rect()
        self.rect.topleft = ((x+1)*FIELD_SIZE) - self.rect.width/2, ((y+1)*FIELD_SIZE) - self.rect.height/2
        self.cur_val = 0
        self.values = values
        self.coords = coords
        self.flow_lines = flow_lines
        self.type = t # s/d

    def collide(self):
        player.klap()
        for f in self.flow_lines:
            if list(f.position) in self.coords or self.env == "g":
                if self.type == 's':
                    if self.cur_val == 0:
                        f.set_speed(self.values[0])
                    else:
                        f.set_speed(self.values[1])
                elif self.type == 'd':
                    for c in range(len(self.coords)):
                        if self.coords[c] == list(f.position):
                            if self.cur_val == 0:
                                f.set_direction(self.values[c][0])
                            else:
                                f.set_direction(self.values[c][1])
        if self.cur_val == 0:
            self.cur_val = 1
            self.image = self.img1.copy()
        else:
            self.cur_val = 0
            self.image = self.img2.copy()
class Checkpoint:
    def __init__(self, x, y, number, state):
        self.img1 = load_image("checkpoint1.png", None, True)
        self.img2 = load_image("checkpoint2.png", None, True)
        self.img3 = load_image("checkpoint3.png", None, True)
        self.rect = self.img1.get_rect()
        self.rect.center = x*FIELD_SIZE+FIELD_SIZE/2, y*FIELD_SIZE+FIELD_SIZE/2
        self.number = number
        if state:
            self.state = CHECKPOINT_READY
        else:
            self.state = CHECKPOINT_DISABLED
        self.set_state(self.state)
    def set_state(self, state):
        self.state = state
        if state == CHECKPOINT_DISABLED:
            self.image = self.img1.copy()
        elif state == CHECKPOINT_READY:
            self.image = self.img2.copy()
        elif state == CHECKPOINT_ENABLED:
            self.image = self.img3.copy()

    def get_state(self):
        return self.state

    def get_number(self):
        return self.number
        
class Goal:
    def __init__(self, x, y):
        self.img1 = load_image("goal2.png", None, True)
        self.img2 = load_image("goal.png", None, True)
        self.image = self.img1
        self.rect = self.img1.get_rect()
        self.rect.center = x*FIELD_SIZE+FIELD_SIZE/2, y*FIELD_SIZE+FIELD_SIZE/2
    def set_ready(self):
        self.image = self.img2
class Airport:
    def __init__(self):
        self.screen = pygame.display.set_mode((640,480))
        self.background = self.screen.copy()
        self.font = load_font('ActionMan.ttf', 50)
        self.font_small = load_font('ActionMan.ttf', 32)
        self.font_mini = load_font('ActionMan.ttf', 24)
        self.title = self.font.render("Airport", 1, (220,50,50))
        self.make_bg()
        self.flow_lines = []
        self.walls = []
        
        for y in range(8):
            self.flow_lines.append(FlowLine((1,1+y), 30, (0,1)))
            self.flow_lines.append(FlowLine((2,1+y), 30, (0,-1)))
        
        for y in range(8):
            self.walls.append(Wall(0,1+y, '|'))
            self.walls.append(Wall(1,1+y, '|'))
            self.walls.append(Wall(2,1+y, '|'))
        self.walls.append(Bench(8,1))
        self.walls.append(Bench(9,1))
        self.walls.append(Bench(10,1))
        self.walls.append(Bench(8,8))
        self.walls.append(Bench(9,8))
        self.walls.append(Bench(10,8))
        
        menu = menu_module.Menu([
            ["Single Player", self.checkpoints],
            ["Two players", self.running2],
            ["Three players", self.running3],
            ["Help", self.main_help],
            ["Quit Game", self.end]], False)
        menu.set_pos(185, 150)
        true = True
        clock = pygame.time.Clock()
        while true:
            time = pygame.time.get_ticks()
            clock.tick(30)
            events = pygame.event.get()
            if events:
                menu.update(events)
                for e in events:
                    if e.type == pygame.QUIT:
                        return
                    elif e.type == KEYDOWN:
                        if e.key == K_ESCAPE:
                            return
                if self.screen.get_width() < 640 or self.screen.get_height() < 480:
                    pygame.display.set_mode((640,480))
                    self.make_bg()
            for f in range(len(self.flow_lines)):
                self.flow_lines[f].update(time)
            self.screen.blit(self.background, (0,0))
            self.screen.blit(self.title, (175, 75))
            for f in range(len(self.flow_lines)):
                self.screen.blit(self.flow_lines[f].image, self.flow_lines[f].rect)
            for w in range(len(self.walls)):
                self.screen.blit(self.walls[w].image, self.walls[w].rect)
            menu.draw(self.screen)
            pygame.display.flip()
    def make_bg(self):
        brick = load_image("texture.png")
        w = self.screen.get_width()/brick.get_width() + 1
        h = self.screen.get_height()/brick.get_height() + 1
        self.background = self.screen.copy()
        for x in range(w):
            for y in range(h):
                self.background.blit(brick, (x*brick.get_width(), y*brick.get_height()))
        self.background.blit(load_image("pruh.png"),(530,0))
        self.screen.blit(self.background, (0,0))
    
    def wait(self, seconds):
        time_limit = pygame.time.get_ticks() + seconds*1000
        clock = pygame.time.Clock()
        while pygame.time.get_ticks() <= time_limit:
            clock.tick(20)
            for event in pygame.event.get():
                if event.type == QUIT:
                    return
                elif event.type == KEYDOWN and event.key == K_ESCAPE:
                    return
    
    def unlock(self, i):
        if self.done < i:
            save_done(i)
            self.done = i
        self.upgrade = True
        
    def checkpoints(self):
        self.upgrade = False
        self.done = load_done()
        self.screen.blit(self.background, (0,0))
        o = [["Prague", self.map3_1],
            ["Frankfurt", self.map4_1],
            ["John F. Kennedy", self.map5_1],
            ["Hartsfield-Jackson", self.map6_1]]
        options = [["Heathrow", self.map1_1]]
        for x in range(self.done):
            try:
                options.append(o[x])
            except:
                pass
        options.append(["Help", self.checkpoints_help])
        options.append(["Back", self.end])
        menu = menu_module.Menu(options, False)
        menu.set_pos(185, 150)
        true = True
        clock = pygame.time.Clock()
        while true:
            time = pygame.time.get_ticks()
            clock.tick(30)
            events = pygame.event.get()
            if events:
                menu.update(events)
                if self.upgrade:
                    self.checkpoints()
                    break
                for e in events:
                    if e.type == pygame.QUIT:
                        return
                    elif e.type == KEYDOWN:
                        if e.key == K_ESCAPE:
                            return
                if self.screen.get_width() < 640 or self.screen.get_height() < 480:
                    pygame.display.set_mode((640,480))
                    self.make_bg()
            for f in range(len(self.flow_lines)):
                self.flow_lines[f].update(time)
            self.screen.blit(self.background, (0,0))
            self.screen.blit(self.title, (175, 75))
            for f in range(len(self.flow_lines)):
                self.screen.blit(self.flow_lines[f].image, self.flow_lines[f].rect)
            for w in range(len(self.walls)):
                self.screen.blit(self.walls[w].image, self.walls[w].rect)
            menu.draw(self.screen)
            pygame.display.flip()
        
    def running2(self):
        self.screen.blit(self.background, (0,0))
        menu = menu_module.Menu([
            ["Heathrow", self.map1_2],
            ["Frankfurt", self.map2_2],
            ["Prague", self.map3_2],
            ["Frankfurt", self.map4_2],
            ["John F. Kennedy", self.map5_2],
            ["Hartsfield-Jackson", self.map6_2],
            ["Help", self.running_help],
            ["Back", self.end]], False)
        menu.set_pos(185, 150)
        true = True
        clock = pygame.time.Clock()
        while true:
            time = pygame.time.get_ticks()
            clock.tick(30)
            events = pygame.event.get()
            if events:
                menu.update(events)
                for e in events:
                    if e.type == pygame.QUIT:
                        return
                    elif e.type == KEYDOWN:
                        if e.key == K_ESCAPE:
                            return
                if self.screen.get_width() < 640 or self.screen.get_height() < 480:
                    pygame.display.set_mode((640,480))
                    self.make_bg()
            for f in range(len(self.flow_lines)):
                self.flow_lines[f].update(time)
            self.screen.blit(self.background, (0,0))
            self.screen.blit(self.title, (175, 75))
            for f in range(len(self.flow_lines)):
                self.screen.blit(self.flow_lines[f].image, self.flow_lines[f].rect)
            for w in range(len(self.walls)):
                self.screen.blit(self.walls[w].image, self.walls[w].rect)
            menu.draw(self.screen)
            pygame.display.flip()
            
    def running3(self):
        self.screen.blit(self.background, (0,0))
        menu = menu_module.Menu([
            ["Heathrow", self.map1_3],
            ["Frankfurt", self.map2_3],
            ["Prague", self.map3_3],
            ["Frankfurt", self.map4_3],
            ["John F. Kennedy", self.map5_3],
            ["Hartsfield-Jackson", self.map6_3],
            ["Help", self.running_help],
            ["Back", self.end]], False)
        menu.set_pos(185, 150)
        true = True
        clock = pygame.time.Clock()
        while true:
            time = pygame.time.get_ticks()
            clock.tick(30)
            events = pygame.event.get()
            if events:
                menu.update(events)
                for e in events:
                    if e.type == pygame.QUIT:
                        return
                    elif e.type == KEYDOWN:
                        if e.key == K_ESCAPE:
                            return
                if self.screen.get_width() < 640 or self.screen.get_height() < 480:
                    pygame.display.set_mode((640,480))
                    self.make_bg()
            for f in range(len(self.flow_lines)):
                self.flow_lines[f].update(time)
            self.screen.blit(self.background, (0,0))
            self.screen.blit(self.title, (175, 75))
            for f in range(len(self.flow_lines)):
                self.screen.blit(self.flow_lines[f].image, self.flow_lines[f].rect)
            for w in range(len(self.walls)):
                self.screen.blit(self.walls[w].image, self.walls[w].rect)
            menu.draw(self.screen)
            pygame.display.flip()

    def checkpoints_help(self):
        self.screen = pygame.display.set_mode((640,480))
        self.make_bg()
        self.screen.blit(self.font_small.render("HELP", 1, [220,50,50]), (30, 30))
        text = ["In each level your goal is to light all",
                "checkpoints to green and reach the goal!"]
        for x in range(len(text)):
            self.screen.blit(self.font_mini.render(text[x], 1, [0,0,0]), (30, 70+30*x))
        self.screen.blit(load_image("checkpoint1.png", None, True), (45, 170))
        text = ["Red checkpoint is disabled, you can",
                "enable it by switching others"]
        for x in range(len(text)):
            self.screen.blit(self.font_mini.render(text[x], 1, [0,0,0]), (95, 155+30*x))
        self.screen.blit(load_image("checkpoint2.png", None, True), (45, 250))
        text = ["You can switch blue checkpoints to",
                "green by walking over them"]
        for x in range(len(text)):
            self.screen.blit(self.font_mini.render(text[x], 1, [0,0,0]), (95, 235+30*x))
        self.screen.blit(load_image("checkpoint3.png", None, True), (45, 330))
        text = ["When the checkpoint is turned green,",
                "then it's done!"]
        for x in range(len(text)):
            self.screen.blit(self.font_mini.render(text[x], 1, [0,0,0]), (95, 315+30*x))
        self.screen.blit(load_image("goal.png"), (30, 400))
        text = ["When the goal turns green, you",
                "should run into it as soon as",
                "possible, then you win!"]
        for x in range(len(text)):
            self.screen.blit(self.font_mini.render(text[x], 1, [0,0,0]), (95, 395+30*x))
        pygame.display.flip()
        running = True
        clock = pygame.time.Clock()
        while running:
            time = pygame.time.get_ticks()
            clock.tick(FPS)
            for e in pygame.event.get():
                if e.type == pygame.QUIT:
                    running = False
                if e.type == pygame.KEYDOWN:
                    running = False

    def running_help(self):
        self.screen = pygame.display.set_mode((640,480))
        self.make_bg()
        self.screen.blit(self.font_small.render("HELP", 1, [220,50,50]), (30, 30))
        text = ["In multiplayer mode you have to catch",
                "the player, which is chosen by computer.",
                "The player tries to escape, till the time,",
                "is over, then he wins!!"]
        for x in range(len(text)):
            self.screen.blit(self.font_mini.render(text[x], 1, [0,0,0]), (30, 70+30*x))
        pygame.display.flip()
        running = True
        clock = pygame.time.Clock()
        while running:
            time = pygame.time.get_ticks()
            clock.tick(FPS)
            for e in pygame.event.get():
                if e.type == pygame.QUIT:
                    running = False
                if e.type == pygame.KEYDOWN:
                    running = False
        
    def main_help(self):
        self.screen = pygame.display.set_mode((640,480))
        self.make_bg()
        self.screen.blit(self.font_small.render("HELP", 1, [220,50,50]), (30, 30))
        text = ["Welcome to the airport! In this game you",
                "will enjoy lots of fun with running,",
                "switching, catching and running again!"]
        for x in range(len(text)):
            self.screen.blit(self.font_mini.render(text[x], 1, [0,0,0]), (30, 70+30*x))
        self.screen.blit(load_image("flow_line.png"), (30, 190))
        text = ["This is flow line, which moves you and",
                "you can use it to speed yourself up"]
        for x in range(len(text)):
            self.screen.blit(self.font_mini.render(text[x], 1, [0,0,0]), (95, 190+30*x))
        self.screen.blit(load_image("paka2.png", None, True), (46, 295))
        text = ["This is switch, hitting it you",
                "switch either speed or direction",
                "of closer flowline"]
        for x in range(len(text)):
            self.screen.blit(self.font_mini.render(text[x], 1, [0,0,0]), (95, 270+30*x))
        self.screen.blit(load_image("bench.png", None, True), (30, 380))
        text = ["The bench is used only as a block",
                "and a decoration"]
        for x in range(len(text)):
            self.screen.blit(self.font_mini.render(text[x], 1, [0,0,0]), (95, 380+30*x))
        pygame.display.flip()
        running = True
        clock = pygame.time.Clock()
        while running:
            time = pygame.time.get_ticks()
            clock.tick(FPS)
            for e in pygame.event.get():
                if e.type == pygame.QUIT:
                    running = False
                if e.type == pygame.KEYDOWN:
                    running = False
    
    def map1_1(self):
        self.run_game("level101.txt", 1, "c", 1)
        
    def map3_1(self):
        self.run_game("level103.txt", 1, "c", 2)
        
    def map4_1(self):
        self.run_game("level104.txt", 1, "c", 3)
        
    def map5_1(self):
        self.run_game("level105.txt", 1, "c", 4)
        
    def map6_1(self):
        self.run_game("level106.txt", 1, "c", 5)
        
    def map1_2(self):
        self.run_game("level001.txt", 2, "r")
        
    def map2_2(self):
        self.run_game("level002.txt", 2, "r")
        
    def map3_2(self):
        self.run_game("level003.txt", 2, "r")
        
    def map4_2(self):
        self.run_game("level004.txt", 2, "r")
        
    def map5_2(self):
        self.run_game("level005.txt", 2, "r")
        
    def map6_2(self):
        self.run_game("level006.txt", 2, "r")

    def map1_3(self):
        self.run_game("level001.txt", 3, "r")
        
    def map2_3(self):
        self.run_game("level002.txt", 3, "r")

    def map3_3(self):
        self.run_game("level003.txt", 3, "r")
        
    def map4_3(self):
        self.run_game("level004.txt", 3, "r")
        
    def map5_3(self):
        self.run_game("level005.txt", 3, "r")

    def map6_3(self):
        self.run_game("level006.txt", 3, "r")
            
    def end(self):
        pygame.event.post(pygame.event.Event(pygame.QUIT,{}))
    
    def run_game(self, level_name, player_number=PLAYERS, mode="c", level_id=None):
        w, f, width, height, extra = load_level(level_name)
        mapsize = (FIELD_SIZE*(width+1),FIELD_SIZE*(height+1))
        self.screen = pygame.display.set_mode((FIELD_SIZE*(width+1)+110,FIELD_SIZE*(height+1)))
        walls = []
        flow_lines = []
        switches = []
        players = []
        start_point = extra["start"]
        rects = []
        tmp_rects = []
        checkpoints = []
        goals = []
        start_points = []
        time_limit = extra["time"]
        for wall in w:
            walls.append(Wall(wall[0], wall[1], wall[2]))
        for b in range(len(extra["bench"])):
            walls.append(Bench(extra["bench"][b][0],extra["bench"][b][1]))
        
        directions = {"^":(0,-1),"<":(-1,0),"v":(0,1),">":(1,0)}
        for flow_line in f:
            fspeed = FLOW_LINE_SPEED
            if extra["fspeed"] != -1:
                fspeed = extra["fspeed"]  
            flow_lines.append(FlowLine((flow_line[0], flow_line[1]), fspeed, copy.deepcopy(directions[flow_line[2]])))
            
        for s in extra["special"]:
            switches.append(Switch(s[0], s[1], extra["switch"][s[2]][0], extra["switch"][s[2]][1], extra["switch"][s[2]][2], flow_lines))

        if mode == 'r':
            hunted = random.randint(0,player_number-1)

        if mode == 'c':
            actual_checkpoint = 0
            first = True
            for x in range(10):
                if extra["checkpoints"].has_key(x):
                    for y in range(len(extra["checkpoints"][x])):
                        checkpoints.append(Checkpoint(extra["checkpoints"][x][y][0], extra["checkpoints"][x][y][1], x, first))
                else:
                    break
                first = False
            for x in range(len(extra["goals"])):
                goals.append(Goal(extra["goals"][x][0],extra["goals"][x][1]))
        for x in range(player_number):
            if mode == 'r':
                start_point = (random.randint(0, width), random.randint(0, height))
            players.append(Player(mapsize, start_point, walls, flow_lines, switches, KEYS[x], COLOURS[x], NICKS[x], x))
        flip = True
        running = True
        clock = pygame.time.Clock()
        
        self.background = self.screen.copy()
        brick = load_image("texture.png")
        w = self.screen.get_width()/brick.get_width() + 1
        h = self.screen.get_height()/brick.get_height() + 1
        self.background = self.screen.copy()
        for x in range(w):
            for y in range(h):
                self.background.blit(brick, (x*brick.get_width(), y*brick.get_height()))
        self.background.blit(load_image("pruh.png"),(FIELD_SIZE*(width+1),0))
        self.screen.blit(self.background, (0,0))
        start_time = pygame.time.get_ticks()
        first_round = True
        while running:
            time = pygame.time.get_ticks()
            clock.tick(FPS)
            for e in pygame.event.get():
                if e.type == pygame.QUIT:
                    running = False
                if e.type == pygame.KEYDOWN and e.key == pygame.K_ESCAPE:
                    running = False
            key = pygame.key.get_pressed()
            for p in range(player_number):
                key_list = [key[players[p].key[0]],key[players[p].key[1]],key[players[p].key[2]],key[players[p].key[3]]]
                players[p].update(key_list, time)

            for f in range(len(flow_lines)):
                flow_lines[f].update(time)
            if mode == 'r':
                for p in range(player_number):
                    if p == hunted:
                        continue
                    if players[p].rect.colliderect(players[hunted]):
                        wanted = load_image("wanted.png", None, True)
                        wanted.blit(self.font_small.render("YOU WIN!!", 1, [0,0,0]), (40, 30))
                        i = 0
                        for p in range(len(players)):
                            if p != hunted:
                                if len(players) == 2:
                                    wanted.blit(players[p].img_list[0][0], (90,90))
                                else:
                                    wanted.blit(players[p].img_list[0][0], (60+i*40,90))
                                i += 1
                        self.screen.blit(wanted, (self.screen.get_width()/2-wanted.get_width()/2, self.screen.get_height()/2-wanted.get_height()/2))
                        pygame.display.flip()
                        self.wait(5)
                        self.screen.blit(self.background, (0,0))
                        running = False
            if mode == 'c':
                if actual_checkpoint != -1:
                    for c in range(len(checkpoints)):
                        if checkpoints[c].rect.colliderect(players[0].rect):
                            if checkpoints[c].number == actual_checkpoint and checkpoints[c].state == CHECKPOINT_READY:
                                player.cink()
                                checkpoints[c].set_state(CHECKPOINT_ENABLED)
                                ready = True
                                for d in range(len(checkpoints)):
                                    if checkpoints[d].get_number() == checkpoints[c].get_number() and checkpoints[d].get_state() != CHECKPOINT_ENABLED:
                                        ready = False
                                        break
                                if ready:
                                    actual_checkpoint += 1
                                    found = False
                                    for d in range(len(checkpoints)):
                                        if checkpoints[d].get_number() == actual_checkpoint:
                                            checkpoints[d].set_state(CHECKPOINT_READY)
                                            found = True
                                    if not found:
                                        actual_checkpoint = -1
                                        for g in range(len(goals)):
                                            goals[g].set_ready()
                                break
                else:
                    for g in range(len(goals)):
                        if goals[g].rect.colliderect(players[0].rect):
                            wanted = load_image("wanted.png", None, True)
                            amount = 5
                            if level_id == LEVEL_NUMBER:
                                wanted.blit(self.font_small.render("CONGRATS!", 1, [0,0,0]), (15, 45))
                                wanted.blit(self.font_small.render("YOU FINISHED", 1, [0,0,0]), (5, 85))
                                wanted.blit(self.font_small.render("THE GAME!!!", 1, [0,0,0]), (15, 125))
                                amount = 10
                            else:
                                self.unlock(level_id)
                                wanted.blit(self.font_small.render("YOU WON!!", 1, [0,0,0]), (40, 90))
                            self.screen.blit(wanted, (self.screen.get_width()/2-wanted.get_width()/2, self.screen.get_height()/2-wanted.get_height()/2))
                            pygame.display.flip()
                            self.wait(amount)
                            self.screen.blit(self.background, (0,0))
                            running = False
                if (time_limit - (time-start_time)/1000) <= 0:
                    wanted = load_image("wanted.png", None, True)
                    wanted.blit(self.font_small.render("YOU LOST!!", 1, [0,0,0]), (15, 50))
                    wanted.blit(self.font_small.render("LET'S TRY IT", 1, [0,0,0]), (15, 90))
                    wanted.blit(self.font_small.render("AGAIN", 1, [0,0,0]), (15, 130))
                    self.screen.blit(wanted, (self.screen.get_width()/2-wanted.get_width()/2, self.screen.get_height()/2-wanted.get_height()/2))
                    pygame.display.flip()
                    self.wait(5)
                    self.screen.blit(self.background, (0,0))
                    running = False
            # RENDERING
            for rect in rects:
                self.screen.blit(self.background.subsurface(rect), rect.topleft)
            if mode == 'c':
                for c in range(len(checkpoints)):
                    self.screen.blit(checkpoints[c].image, checkpoints[c].rect)
                    tmp_rects.append(checkpoints[c].rect)
                for g in range(len(goals)):
                    self.screen.blit(goals[g].image, goals[g].rect)
                    tmp_rects.append(goals[g].rect)
            for f in range(len(flow_lines)):
                self.screen.blit(flow_lines[f].image, flow_lines[f].rect)
                tmp_rects.append(flow_lines[f].rect)
            for w in range(len(walls)):
                self.screen.blit(walls[w].image, walls[w].rect)
                tmp_rects.append(walls[w].rect)
            for s in range(len(switches)):
                self.screen.blit(switches[s].image, switches[s].rect)
                tmp_rects.append(switches[s].rect)
            for p in range(player_number):
                self.screen.blit(players[p].get_image(), players[p].rect)
                tmp_rects.append(players[p].rect)
            t = time_limit - (time-start_time)/1000
            minutes = str(t%60)
            if len(minutes) == 1:
                minutes = "0"+minutes
            s = str(t/60)+":"+minutes
            time_stamp = self.font_small.render(s, 1, [70, 179, 243])
            self.screen.blit(time_stamp,(self.screen.get_width()-170,5))
            time_rect = time_stamp.get_rect()
            time_rect.topleft = (self.screen.get_width()-170,5)
            tmp_rects.append(time_rect)
            rects += tmp_rects
            if flip:
                pygame.display.flip()
                flip = False
            else:
                pygame.display.update(rects)
            rects = copy.deepcopy(tmp_rects)
            tmp_rects = []
            if mode == "r":
                if (time_limit - (time-start_time)/1000) <= 0:
                    wanted = load_image("wanted.png", None, True)
                    wanted.blit(self.font_small.render("ESCAPED!!", 1, [0,0,0]), (40, 30))
                    wanted.blit(players[hunted].img_list[0][0], (90,90))
                    self.screen.blit(wanted, (self.screen.get_width()/2-wanted.get_width()/2, self.screen.get_height()/2-wanted.get_height()/2))
                    pygame.display.flip()
                    self.wait(5)
                    self.screen.blit(self.background, (0,0))
                    running = False
            if mode == "r" and first_round:
                first_round = False
                wanted = load_image("wanted.png", None, True)
                wanted.blit(self.font_small.render("WANTED", 1, [0,0,0]), (40, 30))
                wanted.blit(players[hunted].img_list[0][0], (90,90))
                self.screen.blit(wanted, (self.screen.get_width()/2-wanted.get_width()/2, self.screen.get_height()/2-wanted.get_height()/2))
                pygame.display.flip()
                self.wait(5)
                self.screen.blit(self.background, (0,0))
                flip = True
                start_time = pygame.time.get_ticks()

def main():
    os.environ["SDL_VIDEO_CENTERED"] = "1"
    pygame.init()
    player.play()
    Airport()
    pygame.quit()
    
if __name__ == "__main__":
    os.environ["SDL_VIDEO_CENTERED"] = "1"
    pygame.init()
    player.play()
    Airport()
    pygame.quit()
