import os, pygame, copy

def load_done():
    fullname = os.path.join('data', 'files', 'done.txt')
    try:
        f = file(fullname, 'r')
    except IOError:
        raise SystemExit, "Couldn't load file "+fullname
    for line in f:
        x = line.split()
        return int(x[0])
        
def save_done(i):
    fullname = os.path.join('data', 'files', 'done.txt')
    try:
        f = file(fullname, 'w')
    except IOError:
        raise SystemExit, "Couldn't load file "+fullname
    f.write(str(i))
    f.close() 

def load_music(name):
    fullname = os.path.join('data', 'music', name)
    try:
        sound = pygame.mixer.Sound(fullname)
    except pygame.error, message:
        print 'Cannot load music:', name
        raise SystemExit, message
    return sound

def load_image(name, colorkey=None, alpha=False):
    fullname = os.path.join('data', 'images', name)
    try:
        image = pygame.image.load(fullname)
    except pygame.error, message:
        print 'Cannot load image:', name
        raise SystemExit, message
    if alpha:
        image = image.convert_alpha()
    else:
        image = image.convert()
    if colorkey is not None:
        if colorkey is -1:
            colorkey = image.get_at((0,0))
        image.set_colorkey(colorkey, RLEACCEL)
    return image

def load_font(path, size):
    fullname = os.path.join('data', 'fonts', path)
    return pygame.font.Font(fullname, size)
    
def load_level(name):
    fullname = os.path.join('data', 'levels', name)
    try:
        f = file(fullname, 'r')
    except IOError:
        raise SystemExit, "Couldn't load file "+fullname
    width = y = 0
    l = 0 # 0 = flow_lines, 1 = blocks
    flow_lines = []
    blocks = []
    special_chars = []
    extra = {"fspeed": -1, "switch":{}, "special":[], "checkpoints":{}, "start":[0,0], "goals":[], "bench":[], "time":90}
    for line in f:
        if line[0] == "#":
            line = line.split()
            if line[0] == "#fspeed":
                extra["fspeed"] = int(line[1])
            elif line[0] == "#time":
                extra["time"] = int(line[1])
            elif line[0] == "#switch":
                special_chars.append(line[1])
                if line[2] == "fspeed":
                    coords = []
                    for c in range((len(line)-5)/2):
                        coords.append([int(line[5+c*2]), int(line[6+c*2])])
                    extra["switch"][line[1]] = ["s", copy.deepcopy(coords), [int(line[3]), int(line[4])]]
                elif line[2] == "fdir":
                    coords = []
                    values = []
                    for c in range((len(line)-3)/4):
                        coords.append([int(line[3+c*4]), int(line[4+c*4])])
                        values.append([line[5+c*4], line[6+c*4]])
                    extra["switch"][line[1]] = ["d", copy.deepcopy(coords), copy.deepcopy(values)]
            continue
        for x in range(len(line)):
            if l:
                if x%2 == 0:
                    if line[x] == "-":
                        blocks.append([x/2,y,"-"])
                elif x%2 == 1:
                    if line[x] in special_chars:
                        extra["special"].append([x/2,y,line[x]])
            elif not l:
                if x%2 == 1 and line[x] == "|":
                    blocks.append([x/2,y,"|"])
                elif x%2 == 0:
                    if line[x] in "^<v>":
                        flow_lines.append([x/2,y,line[x]])
                    elif line[x] in "0123456789":
                        if not extra["checkpoints"].has_key(int(line[x])):
                            extra["checkpoints"][int(line[x])] = [[x/2,y]]
                        else:
                            extra["checkpoints"][int(line[x])].append([x/2,y])
                    elif line[x] == "W":
                        extra["bench"].append([x/2,y])
                    elif line[x] == "S":
                        extra["start"] = [x/2,y]
                    elif line[x] == "G":
                        extra["goals"].append([x/2,y])
            width = max(width, x/2)
        if l == 1:
            l = 0
            y += 1
        else:
            l = 1
    return blocks, flow_lines, width, y, extra
    
def load_player_image_list(i):
    imgs = [[],[]]
    if i == 0:
        imgs[0].append(load_image("player1.png", None, True))
        imgs[0].append(load_image("player2.png", None, True))
        imgs[0].append(load_image("player3.png", None, True))
        imgs[0].append(load_image("player4.png", None, True))
        imgs[0].append(load_image("player5.png", None, True))
        imgs[1].append(load_image("player6.png", None, True))
        imgs[1].append(load_image("player7.png", None, True))
        imgs[1].append(load_image("player8.png", None, True))
        imgs[1].append(load_image("player9.png", None, True))
        imgs[1].append(load_image("player10.png", None, True))
    elif i == 1:
        imgs[0].append(load_image("bplayer1.png", None, True))
        imgs[0].append(load_image("bplayer2.png", None, True))
        imgs[0].append(load_image("bplayer3.png", None, True))
        imgs[0].append(load_image("bplayer4.png", None, True))
        imgs[0].append(load_image("bplayer5.png", None, True))
        imgs[1].append(load_image("bplayer6.png", None, True))
        imgs[1].append(load_image("bplayer7.png", None, True))
        imgs[1].append(load_image("bplayer8.png", None, True))
        imgs[1].append(load_image("bplayer9.png", None, True))
        imgs[1].append(load_image("bplayer10.png", None, True))
    else:
        imgs[0].append(load_image("cplayer1.png", None, True))
        imgs[0].append(load_image("cplayer2.png", None, True))
        imgs[0].append(load_image("cplayer3.png", None, True))
        imgs[0].append(load_image("cplayer4.png", None, True))
        imgs[0].append(load_image("cplayer5.png", None, True))
        imgs[1].append(load_image("cplayer6.png", None, True))
        imgs[1].append(load_image("cplayer7.png", None, True))
        imgs[1].append(load_image("cplayer8.png", None, True))
        imgs[1].append(load_image("cplayer9.png", None, True))
        imgs[1].append(load_image("cplayer10.png", None, True))
    for x in range(6):
        if not x%2:
            imgs.append([])
            imgs[-1].append(pygame.transform.rotate(imgs[0][0], (x+2)*45))
            imgs[-1].append(pygame.transform.rotate(imgs[0][1], (x+2)*45))
            imgs[-1].append(pygame.transform.rotate(imgs[0][2], (x+2)*45))
            imgs[-1].append(pygame.transform.rotate(imgs[0][3], (x+2)*45))
            imgs[-1].append(pygame.transform.rotate(imgs[0][4], (x+2)*45))
        else:
            imgs.append([])
            imgs[-1].append(pygame.transform.rotate(imgs[1][0], (x+1)*45))
            imgs[-1].append(pygame.transform.rotate(imgs[1][1], (x+1)*45))
            imgs[-1].append(pygame.transform.rotate(imgs[1][2], (x+1)*45))
            imgs[-1].append(pygame.transform.rotate(imgs[1][3], (x+1)*45))
            imgs[-1].append(pygame.transform.rotate(imgs[1][4], (x+1)*45))
    return imgs